==> 0.0.5.tar.gz: lib/src/field_editor.dart <==
library fields.field_editor;

import 'dart:async';
import 'dart:mirrors';
import 'field.dart';
import 'field_transform.dart';
import 'mirrors.dart';
import 'theme.dart';

class FieldEditorResources {
  String get style =>
"""
input {
  background-color:transparent;
  border-style:none;
  color:${fieldsTheme.colors.text};
}
input[type="text"] {  
  width:100%;
}
input[type="text"][disabled] { 
  color:${fieldsTheme.colors.textDisabled};
}
input[class=error] {
  border: 2px solid ${fieldsTheme.colors.error};
}
""";
  const FieldEditorResources();
}

abstract class FieldEditor {
  static final _fieldEditorMirror = reflectClass(FieldEditor);
  static final _editors = new Map<Type, ClassMirror>();
  static ClassMirror _default;
    
  static bool isRegistered(Type fieldType) => _editors.containsKey(fieldType);
  static bool get isDefaultRegistered => _default != null;
  
  static void register(Type fieldType, Type editorType) {
    if (editorType == null) 
      throw new ArgumentError('editorType must be !null');
    if (isRegistered(fieldType)) 
      throw new StateError('$fieldType already has a registered editorType');    
    final editorClassMirror = reflectClass(editorType);
    if (!isAssignableFrom(_fieldEditorMirror, editorClassMirror))
      throw new ArgumentError(
          'editorType must extend or implement FieldEditor');
    _editors[fieldType] = editorClassMirror;
  }
  
  static void registerDefault(Type editorType) {
    if (editorType == null) 
      throw new ArgumentError('editorType must be !null');
    if (isDefaultRegistered) 
      throw new StateError('Default editorType is already registered.');
    final editorClassMirror = reflectClass(editorType);
    if (!isAssignableFrom(_fieldEditorMirror, editorClassMirror))
      throw new ArgumentError(
          'editorType must extend or implement FieldEditor');
    _default = editorClassMirror;
  }
  
  static bool unregister(Type fieldType) => _editors.remove(fieldType) != null;
  
  static void unregisterDefault() {
    _default = null;
  }
  
  static void unregisterAll() {
    unregisterDefault();
    _editors.clear();
  }
    
  /*Element*/ get element;
  
  final Field field;
  final FieldEditorResources resources;
  
  // TODO(rms): return `T` and make this class generic so that subclasses no
  // longer need to implement this once the following bug is fixed:
  // https://code.google.com/p/dart/issues/detail?id=11345
  Type get valueType;
  
  bool get isValueReadOnly => 
      field.isReadOnly || !canTransformField(valueType, field.type);
  
  factory FieldEditor.forField(Field field) {
    ClassMirror mirror = _default;
    if (field.editor != null)
      mirror = field.editor;
    else if (isRegistered(field.type)) 
      mirror = _editors[field.type];
    if (mirror == null) 
      throw new StateError('No editor registered for type <${field.type}>');
    return mirror.newInstance(const Symbol(''),[field]).reflectee;
  }
  
  FieldEditor(this.field, [this.resources = const FieldEditorResources()]);
    
  Future getValue() { 
    return field.getValue().then((value) {
      if(canTransformField(field.type, valueType))
        return new Future.value(transformField(value, field.type, valueType));
      else return new Future.value(value.toString());
    });
  }

  Future setValue(value) {
    if(isValueReadOnly) {
      return new Future.error(
          new UnsupportedError('Cannot set a read-only value'));
    }
    var transformed;
    try {
      transformed = transformField(value, valueType, field.type);      
    } on FieldTransformError catch(e) {
      return new Future.error(e);
    }
    return field.setValue(transformed);
  }
}

==> 0.0.7.tar.gz: lib/src/field_editor.dart <==
library fields.field_editor;

import 'dart:async';
import 'dart:mirrors';
import 'field.dart';
import 'field_transform.dart';
import 'mirrors.dart';
import 'theme.dart';

class FieldEditorResources {
  String get style =>
"""
input {
  background-color:transparent;
  border-style:none;
  color:${fieldsTheme.colors.text};
}
input[type="text"] {  
  width:100%;
}
input[type="text"][disabled] { 
  color:${fieldsTheme.colors.textDisabled};
}
input[class=error] {
  border: 2px solid ${fieldsTheme.colors.error};
}
""";
  const FieldEditorResources();
}

abstract class FieldEditor {
  static final _fieldEditorClassMirror = reflectClass(FieldEditor);
  static final _editors = new Map<Type, ClassMirror>();
  static ClassMirror _default;
    
  static bool isRegistered(Type fieldType) => _editors.containsKey(fieldType);
  static bool get isDefaultRegistered => _default != null;
  
  static void register(Type fieldType, Type editorType) {
    if (editorType == null) {
      throw new ArgumentError('editorType must be !null');
    }
    if (isRegistered(fieldType)) {
      throw new StateError('$fieldType already has a registered editorType');
    }
    final editorClassMirror = reflectClass(editorType);
    if (!isAssignableFrom(_fieldEditorClassMirror, editorClassMirror)) {
      throw new ArgumentError(
          '$editorType must extend or implement $FieldEditor');
    }
    _editors[fieldType] = editorClassMirror;
  }
  
  static void registerDefault(Type editorType) {
    if (editorType == null) {
      throw new ArgumentError('editorType must be !null');
    }
    if (isDefaultRegistered) {
      throw new StateError('Default editorType is already registered.');
    }
    final editorClassMirror = reflectClass(editorType);
    if (!isAssignableFrom(_fieldEditorClassMirror, editorClassMirror)) {
      throw new ArgumentError(
          '$editorType must extend or implement $FieldEditor');
    }
    _default = editorClassMirror;
  }
  
  static bool unregister(Type fieldType) => _editors.remove(fieldType) != null;
  
  static void unregisterDefault() {
    _default = null;
  }
  
  static void unregisterAll() {
    unregisterDefault();
    _editors.clear();
  }
    
  /*Element*/ get element;
  
  final Field field;
  final FieldEditorResources resources;
  
  // TODO(rms): return `T` and make this class generic so that subclasses no
  // longer need to implement this once the following bug is fixed:
  // https://code.google.com/p/dart/issues/detail?id=11345
  Type get valueType;
  
  bool get isValueReadOnly => 
      field.isReadOnly || !canTransformField(valueType, field.type);
  
  factory FieldEditor.forField(Field field) {
    ClassMirror mirror = _default;
    if (field.editor != null) {
      mirror = field.editor;
    } else if (isRegistered(field.type)) {
      mirror = _editors[field.type];
    }
    if (mirror == null) {
      throw new StateError('No editor registered for type <${field.type}>');
    }
    return mirror.newInstance(const Symbol(''),[field]).reflectee;
  }
  
  FieldEditor(this.field, [this.resources = const FieldEditorResources()]);
    
  Future getValue() { 
    return field.getValue().then((value) {
      if(canTransformField(field.type, valueType)) {
        return new Future.value(transformField(value, field.type, valueType));
      } else {
        return new Future.value(value.toString());
      }
    });
  }

  Future setValue(value) {
    if(isValueReadOnly) {
      return new Future.error(
          new UnsupportedError('Cannot set a read-only value'));
    }
    var transformed;
    try {
      transformed = transformField(value, valueType, field.type);      
    } on FieldTransformError catch(e) {
      return new Future.error(e);
    }
    return field.setValue(transformed);
  }
}
